package com.factory.service.impl;

import com.factory.base.entity.task.*;
import com.factory.mapper.OnsiteTaskMapper;
import com.factory.service.MsgService;
import com.factory.service.OnsiteTaskService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service("onsiteTaskServiceImp")
public class OnsiteTaskServiceImp implements OnsiteTaskService {


    @Autowired
    OnsiteTaskMapper onsiteTaskMapper;
    @Autowired
    private MsgService msgService;

    public int getTaskExistCount(Map<String,Object> map) {
        return onsiteTaskMapper.getTaskExistCount(map);
    }

    public int delTaskOld(Map<String,Object> map) {
        return onsiteTaskMapper.delTaskOld(map);
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public int insertOnsiteTask(OnsiteTask task) {
        return onsiteTaskMapper.insertOnsiteTask(task);
    }


    /**
     * 获取对应的规则
     *
     * @param paarm
     * @return
     */
    @Override
    public List<OnsiteCalendar> getCalenderOnsitesForType(Map<String, Object> param) {
        return onsiteTaskMapper.getCalenderOnsitesForType(param);
    }

    /**
     * 需要提醒的工作
     *
     * @param param
     * @return
     */
    public List<OnsiteCalendar> getRemindWork(Map<String, Object> param) {
        return onsiteTaskMapper.getRemindWork(param);
    }

    /**
     * 更新提醒状态，不太好的写法但是符合现在的需求
     *
     * @param param
     * @return
     */
    public int updateOnsiteRemindFlag(Map<String, Object> param) {
        return onsiteTaskMapper.updateOnsiteRemindFlag(param);
    }


    /**
     * 获取需要提醒人
     *
     * @param map
     * @return
     */
    public Long getRemindNoworkToManager(Map<String,Object> map) {
        return onsiteTaskMapper.getRemindNoworkToManager(map);
    }

    /**
     * 添加不工作提醒信息
     *
     * @param map
     * @return
     */
    @Transactional(rollbackFor = Throwable.class)
    public int addRemindNowork(Map<String,Object> map) {
        int result = onsiteTaskMapper.addRemindNowork(map);
        onsiteTaskMapper.updateOnsiteRemindFlag(map);
        return result;

    }


    /**
     * 获取最大审批级别
     *
     * @param map
     * @return
     */
    public int getMaxApproveLevel(Map<String,Object> map) {
        return onsiteTaskMapper.getMaxApproveLevel(map);
    }

    /**
     * 或许超时没处理记录
     *
     * @param map
     * @return
     */
    public List<ApproveEntry> getApproveTaskNoDeal(Map<String,Object> map) {
        return onsiteTaskMapper.getApproveTaskNoDeal(map);
    }

    @Transactional(rollbackFor = Throwable.class)
    public int addTaskForNext(ApproveEntry vo) {
        vo.setStatus(2);
        vo.setApproval_user_id(1l);
        
        HashMap<String,Object> map = new HashMap<>();
        map.put("approvalLevel", vo.getApproval_level());
        ApprovalUserRes nextUser = onsiteTaskMapper.getApprovalNextUser(map);
        if (nextUser != null) {
            int result = onsiteTaskMapper.updateApproveTask(vo);
            if (result > 0) {
                ApproveEntry approve = new ApproveEntry();
                approve.setTask_id(vo.getTask_id());
                approve.setApproval_level(nextUser.getApproval_level());
                approve.setApproval_role(nextUser.getRole_id());
                int status = onsiteTaskMapper.addApproveTask(approve);
                if (status > 0) {
                    Date date = new Date();
                    SimpleDateFormat ft = new SimpleDateFormat("yyyy-MM-dd");
                    //发送微信消息
                    SendMsgReq sendMsgReq = new SendMsgReq();
                    sendMsgReq.setToUserId(nextUser.getUser_id());
                    sendMsgReq.setTitle("计划放行提醒");
                    sendMsgReq.setContent("您有待放行的计划审批，请登录小程序查看【计划放行】");
                    sendMsgReq.setEndTime(ft.format(date));
                    msgService.sendWechatMessageByRole(sendMsgReq,nextUser.getRole_id());
                }
                return status;
            }
        }
        return 0;
    }

}
